"""__version__.py: Metadata about the pyfolioclient package."""

__title__ = "pyfolioclient"
__description__ = "A Python client for interacting with FOLIO's APIs"
__url__ = "https://github.com/balljok/pyfolioclient"
__version__ = "0.1.2"
